// -*-c++-*-

/*!
  \file debug_server.h
  \brief Debug Server class Header File.
*/

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef SOCCERWINDOW2_DEBUG_SERVER_H
#define SOCCERWINDOW2_DEBUG_SERVER_H

#include <boost/scoped_ptr.hpp>

namespace rcsc {
class UDPSocket;
}
class ViewHolder;

//! monitor client that connect to the rcssserver
class DebugServer
    : public wxEvtHandler {
private:
    boost::scoped_ptr< rcsc::UDPSocket > M_socket;

    //! receive check interval timer.
    wxTimer M_timer;

    //! refence to the view data holder
    ViewHolder & M_view_holder;

    //! not used
    DebugServer();
    //! nocopyable
    DebugServer( const DebugServer & );
    //! nocopyabne
    DebugServer & operator=( const DebugServer & );
public:
    //! constructor
    DebugServer( const int port,
                 ViewHolder & view_holder );

    ~DebugServer();

    //! check if socket is opened.
    bool isConnected() const;

    //! start timer
    void start();
private:

    //! timer event handler
    void handleTimer( wxTimerEvent & event );

};

#endif
